/******************************************************************************/
/*                                                                            */
/*                      Sample PCI - device driver V1.1                       */
/*                                                                            */
/*  Module:        PCI-Device Sample Driver (SAMPLE.TOS)                      */
/*  File:          SAMPLE.C                                                   */
/*  Task:          main function of PCI-device sample driver                  */
/*  Description:   Demo of the usage of some PCI-BIOS functions and how the   */
/*                 BIOS-functions can be used to write an own device driver.  */
/*  Comments:      Before usage of any PCI-BIOS functions, the interface must */
/*                 be initialised with pcibios_init()                         */
/*  Procedures:    main                                                       */
/*                                                                            */
/*  (c)1998-2000 Markus Fichtenbauer                             03.09.2000   */
/*                                                                            */
/******************************************************************************/
/* $Id$ */
/******************************************************************************/
/* $Log$ */
/******************************************************************************/
#include <portab.h>                   /* Portability-declarations (ULONG,...) */
#include <stdio.h>                    /* Standard-I/O (sprintf,...)           */
#include <string.h>                   /* String- und memory functions         */
#include <tos.h>                      /* OS functions (Cconws,...)            */
#include <pci_bios.h>                 /* PCI-BIOS decl. for init interface    */
#include <pci.h>                      /* common PCI declarations              */
#include <debug.h>                    /* declarations for debugging outputs   */

/******************************************************************************/
/*                                                                            */
/*                External Functions / Local Function Prototypes              */
/*                                                                            */
/******************************************************************************/
extern LONG pcibios_init(void);

/******************************************************************************/
/*                                                                            */
/*                             Global Variables                               */
/*                                                                            */
/******************************************************************************/
char  message[100];                   /* string for text outputs with sprintf */

/******************************************************************************/
/*                                                                            */
/*  Main Task:      int main (void)                                           */
/*                                                                            */
/*  Description:    sceleton of a PCI-device driver using ATARI PCI-BIOS      */
/*                                                                            */
/******************************************************************************/
int main(void)
{
  LONG handle;                         /* PCI device handle                   */
  LONG error;                          /* BIOS error codes                    */
  ULONG temp, io_adr, superstack;      /* for temporary data                  */
  UWORD index;                         /* index for searching PCI-devices     */
  UBYTE latency, revision, grant;      /* temporary data for Latency Timer,...*/

  Cconws("\rPCI-device sample driver V1.1\r\n");
  Cconws("in dedication to Florentina\r\n");
  Cconws("\xBD 03.09.2000 by Markus Fichtenbauer\r\n\n");

  Cconws("Initialisation of PCI-BIOS interface...");
  error = pcibios_init();              /* initialisation of BIOS interface    */

  if (error != PCI_SUCCESSFUL)         /* problems during initialisation ?    */
  {
    switch (error)
    {
      case PCI_BIOS_NOT_INSTALLED:
        Cconws("\r\nerror, PCI-BIOS interface not found !\r\n");
        Cconws("\r\nplease use PCI_BIOS.PRG in AUTO-folder\r\n");
      break;
      
      case PCI_BIOS_WRONG_VERSION:
        Cconws("\r\nerror, old (unsupported) PCI-BIOS interface !\r\n");
        Cconws("\r\nplease install new PCI_BIOS.PRG\r\n");
      break;

      default:
        Cconws("unexpected error, Bill Gates is guilty !\r\n");
    }
    Cconws("\r\nPCI-BIOS interface could not be installed !\r\n");
    Cconws("Press any key\r\n");
    Cconin();                          /* wait for key to be pressed          */
    return 0;                          /* without PCI-BIOS I will leave       */
  }
  Cconws("OK\r\n\n");

/******************************************************************************/
/* the pci-bios routines should be called in supervisor-mode                  */
  superstack = Super(0L);              /* supervisor-mode                     */
/******************************************************************************/

/******************************************************************************/
/* example for searching a DEC ethernet card                                  */
/* Vendor-ID: 0x1011, Device-ID: 0x0014                                       */
/******************************************************************************/
  Cconws("EXAMPLE 1: searching for a DEC ethernet card...");
  handle = find_pci_device (0x00141011L, 0);

  if (handle == PCI_DEVICE_NOT_FOUND)
    sprintf(message, "not found\r\n\n\n");
  else
    sprintf(message, "and found (handle = %lx)\r\n\n\n", handle);

  Cconws(message);

/******************************************************************************/
/* example for searching for all possible PCI-devices                         */
/* therefore we use Vendor-ID 0xFFFF that means search all installed devices  */
/******************************************************************************/
  Cconws("EXAMPLE 2: following PCI-devices are found in your 'ATARI':\n\r");
  index = 0;                           /* first PCI-device                    */
    
  do
  {
    handle = find_pci_device (0x0000FFFFL, index++);

    if (handle >= 0)                   /* no error code (errors are negative) */
    {
/*    read Configuration ID of the PCI-device                                 */
      error = read_config_longword(handle, CFID, &temp);

      if (error == PCI_SUCCESSFUL)     /* no errors ?                         */
      {
        sprintf(message, "handle = %ld, Configuration ID: %08lx\r\n", handle, temp);
        Cconws(message);

/*      read the Latency Timer Register                                       */
        latency = fast_read_config_byte(handle, CFHT_LATENCY_TIMER);
        sprintf(message, "Latency Timer = %d bus clocks\r\n", latency);
        Cconws(message);

/*      read the min and max values for the Latency Timer                     */
        latency = fast_read_config_byte(handle, CFIL_MAX_LAT);
        grant = fast_read_config_byte(handle, CFIL_MIN_GNT);
        sprintf(message, "MIN_GNT = %d, MAX_LAT = %d\r\n", grant, latency);
        Cconws(message);

/*      read the Interupt Line Register                                       */
        latency = fast_read_config_byte(handle, CFIL_INTERRUPT_LINE);
        sprintf(message, "Interrupt Line = 0x%02x\r\n", latency);
        Cconws(message);
      }
      else                             /* error occured                       */
      {
        sprintf(message, "handle = %ld, error code = %lx\r\n", handle, error);
        Cconws(message);
      }
    }
    else
      break;

  } while (TRUE);

/******************************************************************************/
/* example for reading values from I/O-space                                  */
/* Vendor-ID: 0x1011, Device-ID: 0x0014                                       */
/******************************************************************************/
  handle = find_pci_device (0x00141011L, 0);
  if (handle >= 0)                     /* no error code (errors are negative) */
  {
    io_adr = fast_read_config_longword(handle, CBA1) & CBA_IOMASK;

/*  now activating the IO-space of the specific card                          */
    write_config_longword(handle, CFCS, CFCS_INIT);
    temp = fast_read_config_longword(handle, CFCS);
    temp |= (CFCR_IOSPACE | CFCR_SERR);
    write_config_word(handle, CFCS_COMMAND, (UWORD)temp);

/*  setting a specific IO-address to an example value of 12345678hex          */
    Cconws("\n\nEXAMPLE 3: writing 0x12345678 to a specific I/O-address...\n\r");
    write_io_longword(handle, io_adr + 0x18, 0x12345678);

/*  reading a longword from the specific IO-address                           */
    temp = fast_read_io_longword(handle, io_adr + 0x18);
    Cconws("reading a longword from that address:\n\r");
TRACEL(temp)                                /* will give 0x12345678           */

/*  reading a word from the specific IO-address                               */
    temp = (ULONG)(fast_read_io_word(handle, io_adr + 0x18));
    Cconws("reading a gword from that address:\n\r");
TRACEL(temp)                                /* will give 0x1234               */

/*  reading a byte from the specific IO-address                               */
    temp = (ULONG)(fast_read_io_byte(handle, io_adr + 0x18));
    Cconws("reading a byte from that address:\n\r");
TRACEL(temp)                                /* will give 0x12                 */
  }

/******************************************************************************/
/* example for searching a Soundblaster PCI64 card                            */
/* Vendor-ID: 0x1274, Device-ID: 0x5000                                       */
/******************************************************************************/
  Cconws("\n\nEXAMPLE 4: searching for a Soundblaster PCI64 card:\r\n");
  handle = find_pci_device (0x50001274L, 0);
  if (handle == PCI_DEVICE_NOT_FOUND)
  {
    sprintf(message, "Soundblaster PCI64 not found\r\n");
    Cconws(message);
  }
  else
  {
    sprintf(message, "SB PCI64 found (handle = %lx)\r\n", handle);
    Cconws(message);

/*  read the Revision ID Register with fast_read_config_byte                  */
    revision = fast_read_config_byte(handle, CFRV_REVISION);
    sprintf(message, "Revision ID = %02x hex\r\n", revision);
    Cconws(message);

/*  read the Revision ID Register with read_config_byte                       */
    read_config_byte(handle, CFRV_REVISION, &revision);
    sprintf(message, "Revision ID = %02x hex\r\n\n", revision);
    Cconws(message);
  }

/* reading the specific machine id of your computer                           */
  temp = get_machine_id();
  Cconws("reading the machine id of your computer:\n\r");
TRACEL(temp)

/******************************************************************************/
/* now at the end of the samples we can switch back to user-mode              */
  Super((void*)superstack);            /* back to User-mode                   */
/******************************************************************************/

  Cconws("\nEnd of demo using the PCI-BIOS interface ;-)\n\r");
  Cconws("Press any key\n\r");
  Cconin();                            /* wait for key to be pressed          */
  return 0;                            /* End of program                      */
}
